#!/bin/bash
# Author : Guth (guth@ploposor.com)
# URL : http://dovecot.org/releases/1.1/dovecot-1.1.8.tar.gz

# program name/version
FilenameFromUrl="$(grep 'U[R]L' $0 | rev | cut -d '/' -f1 | rev)"
softname="$(echo $FilenameFromUrl | cut -d '-' -f1)"
softversion="$(echo $FilenameFromUrl | cut -d '-' -f2 | cut -d '.' -f -3)"
#softversion='1.1.4'
#softname='dovecot'

# architecture/package version
arch='i486'
cpu='i686'
packageversion='54.1'

# number of parallel tasks of make
ntasks='-j2'

# parameters needed for the build process
buildir=$(pwd)
package="$softname-$softversion-$arch-$packageversion"
srcpkg="$buildir/$FilenameFromUrl"

if ! [ -f "$FilenameFromUrl" ]; then
    echo "Downloading the program tarball...$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)"
    wget "$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)" ||exit 0
fi


# prepare the build result directory
dest="$buildir/$package"
rm -rf $dest ||exit 0
rm -f $package.* ||exit 0
mkdir -p $dest ||exit 0
mkdir -p $dest/install ||exit 0
mkdir -p $dest/usr/src/$softname-$softversion ||exit 0

# make a slack-desc file
cat <<EOF > $dest/install/slack-desc
       |-----handy-ruler------------------------------------------------------|
dovecot: dovecot (IMAP / POP3 server written with security primarily in mind)
dovecot:
dovecot: Dovecot is an excellent choice for both small and large installations.
dovecot: It's fast, simple to set up, requires no special administration
dovecot: and it uses very little memory. It's among the highest performing IMAP
dovecot: servers while still supporting the standard mbox and Maildir formats
dovecot: It works with NFS and clustered filesystems and is admin-friendly.
dovecot: User authentication is extremely flexible and feature rich.
dovecot: It supports workarounds  for several bugs in IMAP and POP3 clients
dovecot: Postfix 2.3+ and Exim 4.64+ users can do SMTP authentication directly
dovecot: against Dovecot's authentication backend
EOF

# extract sources
echo "extracting ..."
srcdir="$softname-$softversion"
cd $buildir ||exit 0
rm -rf $srcdir ||exit 0
tar zxf $srcdir.tar.gz ||exit 0
cd $srcdir ||exit 0

# set correct permissions
chown -R root.root .
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;

# configure package for compilation

echo "configuring ..."

mkdir -p $dest/usr/doc/$softname-$softversion

export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"
./configure \
   --prefix=/usr \
   --with-gssapi \
   --with-ldap \
   --with-sql \
   --with-mysql \
   --sysconfdir=/etc \
   --localstatedir=/var \
   --mandir=/usr/man \
   --with-docs \
   --docdir=/usr/doc/$softname-$softversion ||exit 0



for file in $(find ./doc -name Makefile); do
    echo "Patching ${file} docdir value"
    sed "s/\$(datadir)\/doc\/dovecot/\/usr\/doc\/$softname-$softversion/g" ${file} > ${file}_tmp
    mv ${file}_tmp ${file}
done

# compile package
make $ntasks ||exit 0

# install package to the target directory
make install DESTDIR=$dest ||exit 0

# copy the doc files

cp -a \
 AUTHORS COPYING COPYING.LGPL COPYING.MIT ChangeLog INSTALL NEWS README* TODO \
 dovecot-config.in.in dovecot-example.conf \
 doc/dovecot* doc/mkcert.sh doc/securecoding.txt \
 $dest/usr/doc/$softname-$softversion ||exit 0

cp doc/dovecot-openssl.cnf doc/mkcert.sh $dest/etc/ ||exit 0

echo "re-ordering ..."

( cd $dest
    cp -a etc usr/doc/$softname-$softversion/ ||exit 0
    ( cd etc
	mv mkcert.sh dovecot-mkcert.sh.new ||exit 0
	mv dovecot-openssl.cnf dovecot-openssl.cnf.new ||exit 0
	for file in * ; do
	    newfile=`echo ${file} | sed "s/\-example\.conf/\.conf\.new/g"`
	    echo "${file} -> ${newfile}"
	    mv ${file} ${newfile}
	done
    )
)


echo "creating rc script ..."
# put the initialization script in $dest/etc/rc.d/
mkdir -p $dest/etc/rc.d ||exit 0

echo "#!/bin/sh
#
# /etc/rc.d/rc.dovecot
# Start/stop/restart the Dovecot server.
#
# To make dovecot start automatically at boot, make this
# file executable: chmod 755 /etc/rc.d/rc.dovecot
#

case \"\$1\" in
   'start')
      dovecot ;;
   'stop')
      killall dovecot;;
   'restart')
      killall dovecot
      dovecot ;;
   *)
      echo \"usage $0 start|stop|restart\" ;;
esac

" > $dest/etc/rc.d/rc.dovecot.new ||exit 0

chmod 644 $dest/etc/rc.d/rc.dovecot.new ||exit 0
chown root.root $dest/etc/rc.d/rc.dovecot.new ||exit 0

# strip executables
echo "stripping ..."
( cd $dest
   find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
   find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

##No man at all !!!
# gzip man pages and info files
#echo "gzipping ..."
#gzip -9 $dest/usr/man/*/*

# put the ZenWalk build and other needed files in the right directory
echo "setting perms ..."
cp -a $buildir/build-$softname.sh $dest/usr/src/$softname-$softversion ||exit 0
chown -R root.root $dest/usr/src/$softname-$softversion ||exit 0

# change files ownerships for $dest/usr/sbin
# NO $dest/usr/bin at all
#chown -R root.root $dest/usr/bin
chown -R root.root $dest/usr/sbin ||exit 0

# make the package and move it to the build directory

# create doinst.sh file
echo "config() {
  NEW=\"\$1.new\"
  OLD=\"\$1\"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ \"\`cat \$OLD | md5sum\`\" = \"\`cat \$NEW | md5sum\`\" ]; then
    # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for netpkg to consider...
}

mychmod() {
  if [ -a $2.new ]; then
    chmod \$1 \$2.new
  else
    chmod \$1 \$2
  fi
}

config etc/dovecot.conf
config etc/dovecot-ldap.conf
config etc/dovecot-sql.conf
config etc/rc.d/rc.dovecot
config etc/dovecot-openssl.cnf
config etc/dovecot-mkcert.sh
mychmod 700 etc/dovecot-mkcert.sh
mychmod 600 etc/dovecot-openssl.cnf


# check if required user & groups exist &
# otherwise add them
if ! grep -q dovecot /etc/group; then
        groupadd dovecot
fi
if ! grep -q dovecot /etc/passwd; then
        useradd -g dovecot -d /var/run/dovecot -c 'Dovecot' -s /bin/false dovecot
fi

if [ ! -d /var/run/dovecot ] ; then
        mkdir -p /var/run/dovecot
fi
chown dovecot.dovecot /var/run/dovecot

" > $dest/install/doinst.sh

chmod 755 $dest/install/doinst.sh ||exit 0



#------------------( Create The *.dep File )
echo "creating .dep ..."
cd $dest
#requiredbuilder -z $dest > $buildir/$package.dep ||exit 0
echo -n "attr,bzip2,cyrus-sasl,e2fsprogs,glibc,krb5,libcap,libmysqlclient,openldap-client,openssl,pam,samba,zlib" > $buildir/$package.dep ||exit 0

cd $dest
/sbin/makepkg -l y -c n $package.tgz ||exit 0
mv $package.tgz $buildir ||exit 0

# computes the md5sum signature for the package
echo "creating .md5 ..."
cd $buildir ||exit 0
md5sum $package.tgz > $package.md5 ||exit 0

echo "$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)" > $package.src
echo "http://zen-repo.meticul.eu/zen_stuffs/guth/packages/dovecot/build-dovecot.sh" >> $package.src

if [ "$1" == "--cleanup" ]; then
  rm -rf $dest
  rm -rf $srcdir
fi

